@@ -1,34 +1,41 @@
+from __future__ import annotations
+
 import asyncio
 import concurrent.futures
 import io
 import sys
+import warnings
 from collections import deque
-from typing import Deque, Iterable, Optional, Tuple
+from collections.abc import Iterable
 
-from asgiref.typing import (
+from uvicorn._types import (
     ASGIReceiveCallable,
     ASGIReceiveEvent,
     ASGISendCallable,
     ASGISendEvent,
+    Environ,
+    ExcInfo,
     HTTPRequestEvent,
     HTTPResponseBodyEvent,
     HTTPResponseStartEvent,
     HTTPScope,
+    StartResponse,
+    WSGIApp,
 )
 
-from uvicorn._types import Environ, ExcInfo, StartResponse, WSGIApp
 
-
-def build_environ(
-    scope: HTTPScope, message: ASGIReceiveEvent, body: io.BytesIO
-) -> Environ:
+def build_environ(scope: HTTPScope, message: ASGIReceiveEvent, body: io.BytesIO) -> Environ:
     """
     Builds a scope and request message into a WSGI environ object.
     """
+    script_name = scope.get("root_path", "").encode("utf8").decode("latin1")
+    path_info = scope["path"].encode("utf8").decode("latin1")
+    if path_info.startswith(script_name):
+        path_info = path_info[len(script_name) :]
     environ = {
         "REQUEST_METHOD": scope["method"],
-        "SCRIPT_NAME": "",
-        "PATH_INFO": scope["path"].encode("utf8").decode("latin1"),
+        "SCRIPT_NAME": script_name,
+        "PATH_INFO": path_info,
         "QUERY_STRING": scope["query_string"].decode("ascii"),
         "SERVER_PROTOCOL": "HTTP/%s" % scope["http_version"],
         "wsgi.version": (1, 0),
@@ -72,13 +79,21 @@ def build_environ(
     return environ
 
 
-class WSGIMiddleware:
+class _WSGIMiddleware:
     def __init__(self, app: WSGIApp, workers: int = 10):
+        warnings.warn(
+            "Uvicorn's native WSGI implementation is deprecated, you "
+            "should switch to a2wsgi (`pip install a2wsgi`).",
+            DeprecationWarning,
+        )
         self.app = app
         self.executor = concurrent.futures.ThreadPoolExecutor(max_workers=workers)
 
     async def __call__(
-        self, scope: HTTPScope, receive: ASGIReceiveCallable, send: ASGISendCallable
+        self,
+        scope: HTTPScope,
+        receive: ASGIReceiveCallable,
+        send: ASGISendCallable,
     ) -> None:
         assert scope["type"] == "http"
         instance = WSGIResponder(self.app, self.executor, scope)
@@ -98,14 +113,12 @@ class WSGIResponder:
         self.status = None
         self.response_headers = None
         self.send_event = asyncio.Event()
-        self.send_queue: Deque[Optional[ASGISendEvent]] = deque()
+        self.send_queue: deque[ASGISendEvent | None] = deque()
         self.loop: asyncio.AbstractEventLoop = asyncio.get_event_loop()
         self.response_started = False
-        self.exc_info: Optional[ExcInfo] = None
+        self.exc_info: ExcInfo | None = None
 
-    async def __call__(
-        self, receive: ASGIReceiveCallable, send: ASGISendCallable
-    ) -> None:
+    async def __call__(self, receive: ASGIReceiveCallable, send: ASGISendCallable) -> None:
         message: HTTPRequestEvent = await receive()  # type: ignore[assignment]
         body = io.BytesIO(message.get("body", b""))
         more_body = message.get("more_body", False)
@@ -120,9 +133,7 @@ class WSGIResponder:
             body.seek(0)
         environ = build_environ(self.scope, message, body)
         self.loop = asyncio.get_event_loop()
-        wsgi = self.loop.run_in_executor(
-            self.executor, self.wsgi, environ, self.start_response
-        )
+        wsgi = self.loop.run_in_executor(self.executor, self.wsgi, environ, self.start_response)
         sender = self.loop.create_task(self.sender(send))
         try:
             await asyncio.wait_for(wsgi, None)
@@ -147,18 +158,15 @@ class WSGIResponder:
     def start_response(
         self,
         status: str,
-        response_headers: Iterable[Tuple[str, str]],
-        exc_info: Optional[ExcInfo] = None,
+        response_headers: Iterable[tuple[str, str]],
+        exc_info: ExcInfo | None = None,
     ) -> None:
         self.exc_info = exc_info
         if not self.response_started:
             self.response_started = True
             status_code_str, _ = status.split(" ", 1)
             status_code = int(status_code_str)
-            headers = [
-                (name.encode("ascii"), value.encode("ascii"))
-                for name, value in response_headers
-            ]
+            headers = [(name.encode("ascii"), value.encode("ascii")) for name, value in response_headers]
             http_response_start_event: HTTPResponseStartEvent = {
                 "type": "http.response.start",
                 "status": status_code,
@@ -184,3 +192,9 @@ class WSGIResponder:
         }
         self.send_queue.append(empty_body)
         self.loop.call_soon_threadsafe(self.send_event.set)
+
+
+try:
+    from a2wsgi import WSGIMiddleware
+except ModuleNotFoundError:  # pragma: no cover
+    WSGIMiddleware = _WSGIMiddleware  # type: ignore[misc, assignment]
